/*
 * Copyright (C) 2023, KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "configurationmanager.h"
#include <QString>
#include <QSettings>
#include <QFile>
#include <QDir>

static const QString s_autoRotateEnable = "Rotation/autoRotateEnbale";
static const QString s_deviceOrientation = "Rotation/deviceOrientation";
static const QString s_autoSwitchEnable = "DeviceMode/autoSwitchEnable";
static const QString s_tabletMode = "DeviceMode/tabletMode";
static const QString s_tabletModeEnabled = "DeviceMode/tabletModeEnabled";
static const QString s_firstSwitchDeviceMode = "StartupOptions/firstSwitchDeviceMode";
static const QString s_configFilePath = QDir::homePath() + "/.config/statusManager.ini";

ConfigurationManager::ConfigurationManager()
{
    QFile file(s_configFilePath);
    if (!file.exists()) {
        createConfigFile();
    }
    m_qsetting = new QSettings(s_configFilePath, QSettings::IniFormat, this);
}

void ConfigurationManager::createConfigFile()
{
    QSettings *configFile = new QSettings(s_configFilePath, QSettings::IniFormat);
    configFile->setValue("Rotation/autoRotateEnbale",false);
    configFile->setValue("Rotation/deviceOrientation","normal");
    configFile->setValue("DeviceMode/autoSwitchEnable",true);
    configFile->setValue("DeviceMode/tabletMode",false);
    configFile->setValue("StartupOptions/firstSwitchDeviceMode",true);
    configFile->setValue("StartupOptions/firstSwitchDeviceMode",true);
    delete configFile;
}

ConfigurationManager *ConfigurationManager::GetInstance()
{
    static ConfigurationManager s_configurationManager;
    return &s_configurationManager;
}

QString ConfigurationManager::getDeviceOrientation() const
{
    return m_qsetting->value(s_deviceOrientation).toString();
}

void ConfigurationManager::setDeviceOrientation(QString &orientation)
{
    m_qsetting->setValue(s_deviceOrientation, orientation);
}

bool ConfigurationManager::isTabletMode() const
{
    return m_qsetting->value(s_tabletMode).toBool();
}

void ConfigurationManager::setTabletMode(bool tabletMode)
{
    m_qsetting->setValue(s_tabletMode, tabletMode);
}

bool ConfigurationManager::isTabletModeEnabled() const
{
    return m_qsetting->value(s_tabletModeEnabled, false).toBool();
}

void ConfigurationManager::setTabletModeEnabled(bool enabled)
{
    m_qsetting->setValue(s_tabletModeEnabled, enabled);
}

bool ConfigurationManager::isAutoSwitchEnabled() const
{
    return m_qsetting->value(s_autoSwitchEnable).toBool();
}

void ConfigurationManager::setAutoSwitchEnabled(bool autoSwitchEnable)
{
    m_qsetting->setValue(s_autoSwitchEnable, autoSwitchEnable);
}

bool ConfigurationManager::isAutoRotateEnabled() const
{
    return m_qsetting->value(s_autoRotateEnable).toBool();
}

void ConfigurationManager::setAutoRotateEnabled(bool autoRotateEnable)
{
    m_qsetting->setValue(s_autoRotateEnable, autoRotateEnable);
}

bool ConfigurationManager::isFirstSwitchDeviceMode() const
{
    return m_qsetting->value(s_firstSwitchDeviceMode).toBool();
}

void ConfigurationManager::setFirstSwitchDeviceModeToOff()
{
    m_qsetting->setValue(s_firstSwitchDeviceMode, false);
}
